#!/usr/bin/env python

"""
Test document parsing and serialisation.

Copyright (C) 2017, 2018, 2019, 2023 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from os import listdir
from os.path import abspath, split
import sys

# Locate and import the moinformat package.

dirname = split(abspath(sys.argv[0]))[0]
parent = split(dirname)[0]

try:
    import moinformat
except ImportError:
    if "moinformat" in listdir(parent):
        sys.path.append(parent)

# Import specific objects.

from moinformat import get_parser, Metadata, make_input, make_output, \
                       make_parser, make_serialiser, parse, serialise
from moinformat.tree.moin import Container



def test_input(d, s):

    "Compare serialised output from 'd' with its original form 's'."

    metadata = Metadata({
        "pagename" : "TestPage",
        })

    # Encode the input.

    output = make_output(metadata)
    expected = output.encode(s)

    # Obtain and encode the output.

    result = serialise(d, make_serialiser(metadata))
    result = output.encode(result)

    # Test encoded input and output.

    identical = result == expected

    if quiet:
        return identical

    # Show output versus input comparison result.

    print identical
    print "-" * 60
    print result
    if not identical:
        print "-" * 60
        print expected
    print "-" * 60

    # Show HTML serialisation.

    metadata.set("output_format", "html")
    metadata.set("mapping", {"MoinMoin" : "https://moinmo.in/"})

    result = serialise(d, make_serialiser(metadata))
    print output.encode(result)
    print "-" * 60
    print

    return identical

def test_tree(d, t, ts):

    """
    Compare tree structure 'd' with simplified, expected form 't' from 'ts'.
    """

    failing = t.test(d)

    if quiet:
        return not failing

    # Show tree versus expected forms.

    moin_prettyprinter = make_serialiser(Metadata({"input_format" : "moin"}), "pretty")
    tree_prettyprinter = make_serialiser(Metadata({"input_format" : "pretty"}), "pretty")

    print not failing
    print "-" * 60
    print serialise(d, moin_prettyprinter)
    if failing:
        print "-" * 60
        print ts
        simple, tree, error = failing
        print "-" * 60
        print error
        print repr(simple)
        print repr(tree)
        print "-" * 60
        print serialise(tree, tree_prettyprinter)
        print "-" * 60
        print serialise(simple, tree_prettyprinter)
    print "-" * 60
    print

    return not failing

def get_filename(filename):

    "Using 'filename', return the core text filename and any encoding."

    t = filename.split(".")
    if len(t) > 2:
        text_filename = ".".join(t[:2])
        encoding = t[2]
    else:
        text_filename = filename
        encoding = None

    return text_filename, encoding

def get_tree(input, tree_filename):

    "Using 'input', return (text, tree) for 'tree_filename'."

    if input.dir.exists(tree_filename):
        ts = input.readfile(tree_filename)
        return ts, parse(ts, make_parser(Metadata(), "pretty"))
    else:
        return None, None



# Main program.

if __name__ == "__main__":
    args = sys.argv[1:]

    if "--help" in args:
        print >>sys.stderr, """\
Usage: %s [ -q ] [ <filename>... ]

Run the test suite or, if filenames are indicated, specific test files.
The following options are supported:

-q          Suppress test output, reporting only success or failure
--quiet     Equivalent to -q
""" % sys.argv[0]
        sys.exit(1)

    for arg in ["-q", "--quiet"]:
        if arg in args:
            del args[args.index(arg)]
            quiet = True
            break
    else:
        quiet = False

    metadata = Metadata({
        "input_context"     : "directory",
        "input_filename"    : dirname,
        })

    # Make an input context.

    input = make_input(metadata)

    # Obtain input filenames.

    filenames = args or input.dir.select_files("test*.txt*")
    filenames.sort()

    # Process each filename, obtaining a corresponding tree definition.

    for filename in filenames:
        text_filename, encoding = get_filename(filename)

        # Identify any tree-related filenames.

        basename = text_filename.rsplit(".", 1)[0]
        tree_filename = "%s.tree" % basename
        tree_exp_filename = "%s.tree-exp" % basename

        # Read and parse the input.

        s = input.readfile(filename, encoding)
        p = make_parser(metadata)
        d = parse(s, p)

        # Read and parse any tree definitions.

        ts, t = get_tree(input, tree_filename)
        tsexp, texp = get_tree(input, tree_exp_filename)

        # Report the test results.

        if not quiet:
            print filename

        identical = test_input(d, s)
        tree_identical = ts and test_tree(d, t, ts)

        if tsexp:
            p.evaluate_macros()
            tree_exp_identical = test_tree(d, texp, tsexp)
        else:
            tree_exp_identical = None

        if quiet:
            print "%s %s %s: %s" % (identical, tree_identical, tree_exp_identical, filename)

# vim: tabstop=4 expandtab shiftwidth=4
